@@ -429,6 +429,8 @@ typedTerminalImage(TerminalImage ti, EventObj ev)
   { chr = 127;
     if ( valInt(ev->buttons) & BUTTON_meta )
       chr += META_OFFSET;
+  } else if ( ev->id == NAME_DEL )
+  { seq = "\e[3~";
   } else if ( ev->id == NAME_TAB )
   { chr = '\t';
     if ( valInt(ev->buttons) & BUTTON_meta )
@@ -467,6 +469,13 @@ typedTerminalImage(TerminalImage ti, EventObj ev)
   succeed;
 }
 
+static status
+insertSelfTerminalImage(TerminalImage ti, Int chr)
+{ typed_char(ti->data, valInt(chr));
+
+  succeed;
+}
+
 static Name
 getURLTerminalImage(TerminalImage ti, Any from)
 { Int x, y;
@@ -792,6 +801,8 @@ static senddecl send_terminal_image[] =
      NAME_event, "Handle a general event"),
   SM(NAME_typed, 1, "event", typedTerminalImage,
      NAME_event, "Process a keystroke"),
+  SM(NAME_insertSelf, 1, "0..", insertSelfTerminalImage,
+     NAME_event, "Process a keystroke"),
   SM(NAME_copy, 1, "which=[{primary,clipboard}]", copyTerminalImage,
      NAME_selection, "Copy selected text to clipboard or primary selection"),
   SM(NAME_paste, 1, "which=[{primary,clipboard}]", pasteTerminalImage,
